import javax.swing.text.*;
import javax.swing.plaf.TextUI;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.util.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * An text insertion point. The caret maintains two positions:
 * <ol>
 * <li>The dot: The position of the insertion point where the caret is
 *     displayed.</li>
 * <li>The mark: The position that marks the other end of the currently
 *     selected text span.
 * </ol>
 *
 * The span between the dot and the mark is typically highlighted by a
 * Highlighter object.
 *
 * The dot and the mark of the caret will move as the document
 * changes, just like Position objects.
 */
public class MyCaret implements Caret, MouseListener, MouseMotionListener
{
  JTextComponent textComp;
  Object highlight;
  Position posDot;
  Position posMark;
  Rectangle paintedArea;
  Point magicPosition;

  List listeners = new ArrayList();

  /**
   * Install the caret on a text component. This method is typically
   * called by the setCaret() method of the text component.
   */
  public void install(JTextComponent comp)
  {
    textComp = comp;
    textComp.addMouseListener(this);
    textComp.addMouseMotionListener(this);
  }

  public void deinstall(JTextComponent comp)
  {
    textComp.removeMouseListener(this);
    textComp = null;
  }

  public void paint(Graphics gfx)
  {
    TextUI ui = textComp.getUI();
    gfx.setColor(textComp.getCaretColor());
    paintedArea = getCaretArea();
    gfx.drawLine(paintedArea.x, paintedArea.y,
		 paintedArea.x, paintedArea.y+paintedArea.height-1);
  }

  private final void damage(Rectangle area)
  {
    if (area == null) return;

    textComp.repaint(area.x, area.y,
		     area.width, area.height);
  }

  /** 
   * Add a listener that wants to be informed of changes in the caret
   * position. The text component typically listens to the caret to
   * [unknown purpose].  */
  public void addChangeListener(ChangeListener listener)
  {
    listeners.add(listener);
  }

  public void removeChangeListener(ChangeListener listener)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public boolean isVisible()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public void setVisible(boolean visible)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public boolean isSelectionVisible()
  {
    boolean visible = (getDot() != getMark());

    return visible; // always visible
  }

  public void setSelectionVisible(boolean visible)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public void setMagicCaretPosition(Point point)
  {
    magicPosition = point;
  }

  public Point getMagicCaretPosition()
  {
    return magicPosition;
  }

  public void setBlinkRate(int msec)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public int getBlinkRate()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public int getDot()
  {
    int dot = posDot.getOffset();
    return dot;
  }

  public int getMark()
  {
    int mark = posMark.getOffset();
    
    return mark;
  }

  /**
   * Set dot and mark to the given offset. Use moveDot() to only set dot.
   */
  public void setDot(int offset)
  {
    try
      {
	posMark = textComp.getDocument().createPosition(offset);
	moveDot(posMark);
      }
    catch (BadLocationException ble)
      {
	throw new Error(ble.toString());
      }
  }

  public void moveDot(int offset)
  {
    try
      {
	moveDot(textComp.getDocument().createPosition(offset));
      }
    catch (BadLocationException ble)
      {
	throw new Error(ble.toString());
      }
  }

  private void moveDot(Position pos) throws BadLocationException
  {
    posDot = pos;
    int mark = posMark.getOffset();
    int dot = posDot.getOffset();

    // Remove any previously painted caret by damaging the affected area.
    if (paintedArea != null) 
      {
	damage(paintedArea);
	paintedArea = null;
      }

    // Remove, change or update highlight.
    if (dot == mark) 
      {
	if (highlight != null)
	  {
	    textComp.getHighlighter().removeHighlight(highlight);
	    highlight = null;
	  }
      }
    else {
      if (highlight == null) 
	{
	  Highlighter.HighlightPainter hp =
	    MyDefaultHighlighter.DefaultPainter;
	  highlight = textComp.getHighlighter().addHighlight(mark, dot, hp);
	}
      else 
	{
	  textComp.getHighlighter().changeHighlight(highlight, mark, dot);
	}
    } 

    ChangeEvent ce = new ChangeEvent(this);
    Iterator itr = listeners.iterator();
    while (itr.hasNext()) 
      {
	ChangeListener cl = (ChangeListener) itr.next();
	cl.stateChanged(ce);
      }

    damage(getCaretArea());
  }

  /**
   * @returns area of caret, may be null
   */
  private final Rectangle getCaretArea()
  {
    try 
      {
	TextUI ui = textComp.getUI();
	return ui.modelToView(textComp, posDot.getOffset());
      }
    catch(BadLocationException ble) 
      {
	throw new Error(ble.toString());
      }
  }


  // ------- MouseListener implementation:
  
  public void mouseClicked(MouseEvent e)
  {
    System.out.println("click: NOP");
  }

  public void mousePressed(MouseEvent event)
  {
    textComp.requestFocus();

    TextUI ui = textComp.getUI();
    Position.Bias[] biasRef = new Position.Bias[1];
    int offset = ui.viewToModel(textComp, event.getPoint(), biasRef);
    setDot(offset);
  }

  public void mouseReleased(MouseEvent e)
  {
  }

  public void mouseEntered(MouseEvent e)
  {
  }

  public void mouseExited(MouseEvent e)
  {
  }


  // -------- MouseMotionListener implementation:
  
  public void mouseDragged(MouseEvent event)
  {
    TextUI ui = textComp.getUI();
    Position.Bias[] biasRef = new Position.Bias[1];
    int offset = ui.viewToModel(textComp, event.getPoint(), biasRef);
    moveDot(offset);
  }

  public void mouseMoved(MouseEvent e)
  {
  }

}
