
import javax.swing.text.*;
import javax.swing.plaf.TextUI;
import java.awt.Graphics;
import java.awt.Shape;
import java.awt.Color;
import java.awt.Rectangle;
import java.util.*;

public class MyDefaultHighlighter implements Highlighter
{

  static class MyHighlight implements Highlighter.Highlight
  {
    Position start;
    Position end;
    Highlighter.HighlightPainter painter;

    public MyHighlight(Position start, Position end,
		       Highlighter.HighlightPainter painter)
    {
      this.start   = start;
      this.end     = end;
      this.painter = painter;
    }

    public int getStartOffset()
    {
      return start.getOffset();
    }

    public int getEndOffset()
    {
      return end.getOffset();
    }

    public Highlighter.HighlightPainter getPainter()
    {
      return painter;
    }
  }

  public static class DefaultHighlightPainter
      extends LayeredHighlighter.LayerPainter
  {
    Color color;

    public DefaultHighlightPainter(Color c)
    {
      color = c;
    }

    public Color getColor()
    {
      return color;
    }

    /**
     * Paint highlight.
     *
     * @param bounds Indication of the margins of the text area. This
     *		is used when drawing highlights that span multiple lines.
     */

    public void paint(Graphics gfx, int startOffset, int endOffset,
		      Shape bounds, JTextComponent textComp)
    {
      if (gfx != null) return;
      
      TextUI ui = textComp.getUI();
      Rectangle startRect, endRect;
      try 
	{
	  startRect = ui.modelToView(textComp, startOffset);
	  endRect   = ui.modelToView(textComp, endOffset);
	} 
      catch (BadLocationException ble)
	{
	  throw new Error(ble.toString());
	}	  
      /*
	,--- bounds.x
	|<------- bounds.width ------->|
	|          ,--- startRect (x, y)
	|         /	         
	:        +---------------------+ }--- h            }
	+--------+                     | --- middleY       }--- middleH
	|<--------- middleW ---------->|                   }
	|                   +----------+ --- middleBottom  }
	+-------------------+\
	                      `--- endRect (endX, middleBottom)  
      */
      
      Color c = getColor();
      if (c == null)
	c = textComp.getSelectionColor();

      //gfx.setColor(c);
      gfx.setColor(Color.red);
      
      int x = startRect.x;
      int y = startRect.y;
      int endX = endRect.x;
      int h = startRect.height;
      
      if (y == endRect.y) 
	{
	  // Whole highlight on one line
	  gfx.fillRect(x, y, endX - x, h);
	}
      else 
	{
	  Rectangle rectBounds = bounds.getBounds();
	  
	  // Highlight spans multiple lines
	  int left    = rectBounds.x;
	  int middleW = rectBounds.width;
	  
	  gfx.fillRect(x, y, left + middleW - x, h);
	  
	  int middleBottom = endRect.y;
	  int middleY = y + h;
	  int middleH = middleBottom - middleY;
	  
	  if (middleH > 0)
	    gfx.fillRect(left, middleY, middleW, middleH);
	  
	  gfx.fillRect(left, middleBottom, endX - left, endRect.height);
	}
    }
    
    

    public Shape paintLayer(Graphics g, int offs0, int offs1, Shape bounds,
			    JTextComponent c, View view)
    {
      throw new UnsupportedOperationException("Not implemented yet");
    }
  }

  public static DefaultHighlightPainter DefaultPainter =
      new DefaultHighlightPainter(null);
  


  List highlights = new ArrayList();
  JTextComponent textComp;

  /**
   * Install this highlighter on a specific text component. This
   * method is typically called by the setHighligher() method of the
   * text component.
   */
  public void install(JTextComponent comp)
  {
    textComp = comp;
    highlights.clear();
  }

  public void deinstall(JTextComponent comp)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  /**
   * Paint all the registered highlights.
   */
  public void paint(Graphics gfx)
  {
    Shape bounds = textComp.getBounds();
    Iterator itr = highlights.iterator();
    while (itr.hasNext()) 
      {
	MyHighlight h = (MyHighlight) itr.next();
	h.getPainter().paint(gfx, h.getStartOffset(), h.getEndOffset(),
			     bounds, textComp);
      }
  }

  /** 
   * Add a span that should be highlighted. This method is typically
   * called by the caret object, as a result of moving the dot, thus
   * making the span between the dot and the mark selected.
   * 
   * @return A tag that can be used to refer to the highlight.
   */
  public Object addHighlight(int p0, int p1, Highlighter.HighlightPainter hp)
    throws BadLocationException
  {
    Document doc = textComp.getDocument();
    Position start = doc.createPosition(p0);
    Position end   = doc.createPosition(p1);
    MyHighlight h = new MyHighlight(start, end, hp);
    highlights.add(h);
    damageRange(p0, p1);
    return h;
  }

  public void removeHighlight(Object tag)
  {
    MyHighlight h = (MyHighlight) tag;
    highlights.remove(h);
    damageRange(h.getStartOffset(), h.getEndOffset());
  }

  public void removeAllHighlights()
  {
    highlights.clear();
  }

  public void changeHighlight(Object tag, int p0, int p1)
    throws BadLocationException
  {
    Document doc = textComp.getDocument();
    MyHighlight h = (MyHighlight) tag;
    
    damageRange(h.getStartOffset(), h.getEndOffset());
 
    h.start = doc.createPosition(p0);
    h.end   = doc.createPosition(p1);
    
    damageRange(p0, p1);
  }

  static final Highlighter.Highlight[] EMPTY_HIGHLIGHT_ARRAY =
      new Highlighter.Highlight[0];

  public Highlighter.Highlight[] getHighlights()
  {
    return (Highlighter.Highlight[]) highlights.toArray(EMPTY_HIGHLIGHT_ARRAY);
  }


  final void damageRange(int p0, int p1)
  {
    if (p1<p0) 
      {
	int tmp = p0;
	p0 = p1;
	p1 = tmp;
      }
    textComp.getUI().damageRange(textComp, p0, p1);
  }
}
