/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.Serializable;
import javax.swing.plaf.*;
import javax.swing.border.*;


/** A second system AWT Component, ie. bloat, bloat, bloat... */

public abstract class JComponent extends Container implements Serializable
{
  protected transient ComponentUI ui;
  //protected javax.swing.event.EventListenerList listenerList;

  public static final int WHEN_FOCUSED                       =  0;
  public static final int WHEN_ANCESTOR_OF_FOCUSED_COMPONENT =  1;
  public static final int WHEN_IN_FOCUSED_WINDOW             =  2;
  public static final int UNDEFINED_CONDITION                = -1;
  
  public static final String TOOL_TIP_TEXT_KEY = "ToolTipText";
  
  //protected javax.accessibility.AccessibleContext accessibleContext;
  
  Component nextFocusable;
  boolean isRequestFocusEnabled = true;
  boolean verifyInputWhenFocusTarget = true;
  Dimension preferredSize;
  Dimension maxSize;
  Dimension minSize;
  Border border;
  boolean isOpaque = false;
  boolean isDoubleBuffered = false;
  
  private static final int HAS_ALIGNMENT_X = 1;
  private static final int HAS_ALIGNMENT_Y = 2;
  
  int alignFlags = 0;
  float alignX;
  float alignY;
  
  public JComponent()
  {
  }
  
  public void updateUI()
  {
    throw new RuntimeException("updateUI must be overriden");
  }

  protected void setUI(ComponentUI newUI)
  {
    if (ui != null) 
      ui.uninstallUI(this);
    ui = newUI;
    if (ui != null)
      ui.installUI(this);
  }
  
  public String getUIClassID()
  {
    return "ComponentUI";
  }
  
  protected Graphics getComponentGraphics(Graphics gfx)
  {
    // TODO: DebugGraphics option
    gfx.setFont(getFont());
    gfx.setColor(getForeground());
    return gfx;
  }

  /** 
   * Calls upon UI delegate to paint component.  This method calls
   * update() on the UI delegate.  It is up to the update() method of
   * the UI delegate forward the request to UI paint().
   */
  protected void paintComponent(Graphics gfx)
  {
    if (ui != null)
      {
	Graphics gfxCopy = gfx.create();
	ui.update(gfxCopy, this);
      }
  }

  protected void paintChildren(Graphics gfx)
  {
    super.paint(gfx); // FIXME: do anything else?
  }
  
  protected void paintBorder(Graphics gfx)
  {
    Border border = getBorder();
    
    if (border == null)
      return;
    border.paintBorder(this, gfx, 0, 0, getWidth(), getHeight());
  }
  
  public void update(Graphics gfx)
  {
    paint(gfx);
  }

  public void paint(Graphics gfx)
  {
    paintComponent(gfx);
    paintBorder(gfx);
    paintChildren(gfx);
  }

  public void printAll(Graphics gfx)
  {
    print(gfx);
  }

  public void print(Graphics gfx)
  {
    printComponent(gfx);
    printBorder(gfx);
    printChildren(gfx);
  }

  protected void printComponent(Graphics gfx)
  {
    paintComponent(gfx);
  }

  protected void printChildren(Graphics gfx)
  {
    paintChildren(gfx);
  }

  protected void printBorder(Graphics gfx)
  {
    paintBorder(gfx);
  }

  public boolean isPaintingTile()
  {
    return false; // FIXME: How can you tell?
  }
  
  public boolean isFocusCycleRoot()
  {
    return false; // implied default in Javadocs
  }

  public boolean isManagingFocus()
  {
    return false; // implied default in Javadocs
  }
  
  public void setNextFocusableComponent(Component next)
  {
    nextFocusable = next;
  }

  public Component getNextFocusableComponent()
  {
    return nextFocusable;
  }

  public void setRequestFocusEnabled(boolean isEnabled)
  {
    isRequestFocusEnabled = isEnabled;
  }
    
  public boolean isRequestFocusEnabled()
  {
    return isRequestFocusEnabled;
  }

  public void requestFocus()
  {
    if (isRequestFocusEnabled())
      grabFocus();
  }
  
  public void grabFocus()
  {
    if (getVerifyInputWhenFocusTarget())
      {
	// TODO: do verifying
      }

    super.requestFocus();
  }

  public void setVerifyInputWhenFocusTarget(boolean verify)
  {
    verifyInputWhenFocusTarget = verify;
  }

  public boolean getVerifyInputWhenFocusTarget()
  {
    return verifyInputWhenFocusTarget;
  }

  public void setPreferredSize(Dimension size)
  {
    preferredSize = size;
  }

  public Dimension getPreferredSize()
  {
    return (preferredSize != null) ?
      preferredSize :
      ui.getPreferredSize(this);
  }

  public void setMaximumSize(Dimension size)
  {
    maxSize = size;
  }
    
  public Dimension getMaximumSize()
  {
    return (maxSize != null) ?
      preferredSize :
      ui.getMaximumSize(this);
  }

  public void setMinimumSize(Dimension size)
  {
    minSize = size;
  }
  
  public Dimension getMinimumSize()
  {
    return (minSize != null) ?
      preferredSize :
      ui.getMinimumSize(this);
  }
	
  public boolean isMinimumSizeSet()
  {
    return minSize != null;
  }

  public boolean isPreferredSizeSet()
  {
    return preferredSize != null;
  }

  public boolean isMaximumSizeSet()
  {
    return maxSize != null;
  }

  public boolean contains(int x, int y)
  {
    return ui.contains(this, x, y);
  }

  public void setBorder(Border border)
  {
    this.border = border;
    repaint();
  }
  
  public Border getBorder()
  {
    return border;
  }

  public Insets getInsets()
  {
    if (border == null) 
      super.getInsets();
    return border.getBorderInsets(this);
  }
  
  public Insets getInsets(Insets insets)
  {
    return getInsets();
  }

  public float getAlignmentY() 
  {
    if ((alignFlags & HAS_ALIGNMENT_Y) == 0) 
      super.getAlignmentY();
    return alignY;
  }
  
  public void setAlignmentY(float alignmentY) 
  {
    alignY = alignmentY;
    alignFlags |=  HAS_ALIGNMENT_Y;
  }

  public float getAlignmentX() 
  {
    if ((alignFlags & HAS_ALIGNMENT_X) == 0)
      super.getAlignmentX();
    return alignX;
  }
  
  public void setAlignmentX(float alignmentX)
  {
    alignX = alignmentX;
    alignFlags |=  HAS_ALIGNMENT_X;
  }

  //public void setInputVerifier(InputVerifier inputVerifier);
  
  //public InputVerifier getInputVerifier();

  public Graphics getGraphics() 
  {
    return super.getGraphics();
    // TODO: DebugGraphics
  }

  public void setDebugGraphicsOptions(int debugOptions) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }

  public int getDebugGraphicsOptions() 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  /*
  public void registerKeyboardAction(ActionListener anAction,
				     String aCommand,
				     KeyStroke aKeyStroke,
				     int aCondition);
  */
  /*
  public void registerKeyboardAction(ActionListener anAction,
				     KeyStroke aKeyStroke,
				     int aCondition)
  */

  //public void unregisterKeyboardAction(KeyStroke aKeyStroke);

  //public KeyStroke[] getRegisteredKeyStrokes();

  //public int getConditionForKeyStroke(KeyStroke aKeyStroke);

  //public ActionListener getActionForKeyStroke(KeyStroke aKeyStroke);

  public void resetKeyboardActions()
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
	
  //public final void setInputMap(int condition, InputMap map);

  //public final InputMap getInputMap(int condition);

  //public final InputMap getInputMap();

  //public final void setActionMap(ActionMap am);

  //public final ActionMap getActionMap();

  public boolean requestDefaultFocus()
  {
    throw new UnsupportedOperationException("not implemented yet");
  }

  public void setVisible(boolean visible)
  {
    super.setVisible(visible); // not sure what to do here
  }    
  
  /** @deprecated */
  public void hide()
  {
    super.setVisible(false); // not sure what to do here
  }

  public void setEnabled(boolean enabled)
  {
    super.setEnabled(enabled); // not sure what to do here
  }    
  
  public void setForeground(Color fg)
  {
    super.setForeground(fg); // not sure what to do here
  }    

  public void setBackground(Color bg)
  {
    super.setBackground(bg); // not sure what to do here
  }    

  public void setFont(Font font)
  {
    super.setFont(font);
  }

  public boolean isFocusTraversable()
  {
    return super.isFocusTraversable();  // not sure what to do here
  }

  protected void processFocusEvent(FocusEvent evt)
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  protected void processComponentKeyEvent(KeyEvent evt) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  protected void processKeyEvent(KeyEvent evt) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
  
  /*
  protected boolean processKeyBinding(KeyStroke ks,
				      KeyEvent e,
				      int condition,
				      boolean pressed);
  */

  public void setToolTipText(String text)
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  public String getToolTipText()
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
  
  public String getToolTipText(MouseEvent evt)
  {
    throw new UnsupportedOperationException("not implemented yet");
  }

  public Point getToolTipLocation(MouseEvent evt)
  {
    throw new UnsupportedOperationException("not implemented yet");
  }

  //public JToolTip createToolTip();

  public void scrollRectToVisible(Rectangle aRect)
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  public void setAutoscrolls(boolean autoscrolls) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    

  public boolean getAutoscrolls()
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  protected void processMouseMotionEvent(MouseEvent evt) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
    
  public void enable() 
  {
    super.setEnabled(true); // not sure what to do here
  }
    
  public void disable() 
  {
    super.setEnabled(false);
  }

  //public AccessibleContext getAccessibleContext();
  
  public final Object getClientProperty(Object key) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }

  public final void putClientProperty(Object key, Object value) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
	
  public static boolean isLightweightComponent(Component comp) 
  {
    return comp.isLightweight();
  }
    
  public void reshape(int x, int y, int w, int h) 
  {
    setBounds(x, y, w, h); // not sure what to do here
  }

  public Rectangle getBounds(Rectangle rv) 
  {
    return super.getBounds(rv);  // not sure what to do here
  }

  public Dimension getSize(Dimension rv) 
  {
    return super.getSize(rv); // not sure what to do here
  }

  public Point getLocation(Point rv) 
  {
    return super.getLocation(rv); // not sure what to do here
  }

  public int getX() 
  {
    return super.getX();
  }
    
  public int getY() 
  {
    return super.getY();
  }

  public int getWidth() 
  {
    return super.getWidth();
  }

  public int getHeight() 
  {
    return super.getHeight();
  }
    
  public boolean hasFocus() 
  {
    return super.hasFocus();
  }

  public boolean isOpaque() 
  {
    return isOpaque;
  }

  /**
   * Setting this to true will prompt the default ui.update()
   * implementation to clear the background. 
   */
  public void setOpaque(boolean isOpaque)
  {
    this.isOpaque = isOpaque;
  }
  
  public void computeVisibleRect(Rectangle visibleRect) 
  {
    // TODO
  }
    
  public Rectangle getVisibleRect() 
  {
    Rectangle rect = new Rectangle(0, 0, getWidth(), getHeight());
    computeVisibleRect(rect);
    return rect;
  }

  protected void firePropertyChange(String propertyName, Object oldValue,
				    Object newValue) 
  {
    super.firePropertyChange(propertyName, oldValue, newValue);
  }
  
  public void firePropertyChange(String propertyName,
				 byte oldValue,
				 byte newValue) 
  {
    firePropertyChange(propertyName, 
		       new Byte(oldValue), new Byte(newValue));
  }
    
  public void firePropertyChange(String propertyName,
				 char oldValue,
				 char newValue)
  {
    firePropertyChange(propertyName, 
		       new Character(oldValue), new Character(newValue));
  }
  
  public void firePropertyChange(String propertyName,
				 short oldValue,
				 short newValue)
  {
    firePropertyChange(propertyName, 
		       new Short(oldValue), new Short(newValue));
  }
  
  public void firePropertyChange(String propertyName,
				 int oldValue,
				 int newValue) 
  {
    firePropertyChange(propertyName, 
		       new Integer(oldValue), new Integer(newValue));
  }
    
  public void firePropertyChange(String propertyName,
				 long oldValue,
				 long newValue) 
  {
    firePropertyChange(propertyName, 
		       new Long(oldValue), new Long(newValue));
  }
  
  
  public void firePropertyChange(String propertyName,
				 float oldValue,
				 float newValue)
  {
    firePropertyChange(propertyName, 
		       new Float(oldValue), new Float(newValue));
  }
  
  public void firePropertyChange(String propertyName,
				 double oldValue,
				 double newValue)
  {
    firePropertyChange(propertyName, 
		       new Double(oldValue), new Double(newValue));
  }
  
  public void firePropertyChange(String propertyName,
				 boolean oldValue,
				 boolean newValue) 
  {
    firePropertyChange(propertyName, 
		       new Boolean(oldValue), new Boolean(newValue));
  }

  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    super.addPropertyChangeListener(listener);
  }
  
  public void addPropertyChangeListener(String propertyName,
					PropertyChangeListener listener) 
  {
    super.addPropertyChangeListener(propertyName, listener);
  }
    
  public void removePropertyChangeListener(PropertyChangeListener listener) 
  {
    super.removePropertyChangeListener(listener);
  }

  public void removePropertyChangeListener(String propertyName,
					   PropertyChangeListener listener) 
  {
    super.removePropertyChangeListener(propertyName, listener);
  }
  
  protected void fireVetoableChange(String propertyName,
				    Object oldValue,
				    Object newValue
				    ) throws PropertyVetoException 
  {
    throw new UnsupportedOperationException("not implemented");
  }
  
  
  public void addVetoableChangeListener(VetoableChangeListener listener) 
  {
    throw new UnsupportedOperationException("not implemented");
  }
    
  public void removeVetoableChangeListener(VetoableChangeListener listener) 
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public Container getTopLevelAncestor() 
  {
    Container ancestor = this;
    while (!((ancestor == null) ||
	     (ancestor instanceof Window)
	     //|| (ancestor instanceof Applet)
	     )) {
      ancestor = ancestor.getParent();
    }
    return ancestor;
  }
  
  //public void addAncestorListener(AncestorListener listener);
  //public void removeAncestorListener(AncestorListener listener);
  
  //public EventListener[] getListeners(Class listenerType);

  public void addNotify() 
  {
    super.addNotify();
    
    // TODO?
  }
    

  public void removeNotify() 
  {
    super.removeNotify();
  }    

  public void repaint(long tm, int x, int y, int width, int height) 
  {
    super.repaint(tm, x, y, width, height);
  }
  
  public void repaint(Rectangle r) 
  {
    repaint(r.x, r.y, r.width, r.height);
  }
  
  public void revalidate() 
  {
    throw new UnsupportedOperationException("not implemented");
  }
  
  public boolean isValidateRoot() 
  {
    return false;
  }

  public boolean isOptimizedDrawingEnabled() 
  {
    return true;
  }
  
  /** should never be used IMO */
  public void paintImmediately(int x, int y, int w, int h) 
  {
    Graphics gfx = getGraphics();
    Rectangle clip = new Rectangle(x, y, w, h);
    computeVisibleRect(clip);
    
    gfx.setClip(clip);
    paint(gfx);
  }
  
  public void paintImmediately(Rectangle r) 
  {
    Graphics gfx = getGraphics();
    gfx.setClip(r);
    paint(gfx);
  }

  /** FIXME: doesn't actually do anything yet */
  public void setDoubleBuffered(boolean buffer) 
  {
    isDoubleBuffered = buffer;
  }
    
  public boolean isDoubleBuffered() 
  {
    return isDoubleBuffered;
  }
    
  //public JRootPane getRootPane();
    
  protected String paramString() 
  {
    StringBuffer sb = new StringBuffer(super.paramString());
    
    Border border = getBorder();
    if (border != null) 
      {
	sb.append("border=");
	sb.append(border);
      }

    if ((alignFlags & HAS_ALIGNMENT_X) != 0) 
      {
	sb.append("alignx=");
	sb.append(getAlignmentX());
      }
    if ((alignFlags & HAS_ALIGNMENT_Y) != 0) 
      {
	sb.append("yalign=");
	sb.append(getAlignmentY());
      }
    
    return sb.toString();
  }
}
